#include <nvbuf_utils.h>
#include <iostream>
#include <fstream>
#include <cassert>
#include <string>

bool CreateBuffer(uint32_t width, uint32_t height, NvBufferColorFormat fmt, NvBufferParams& buffer){
  NvBufferCreateParams params{};
  params.width = width;
  params.height = height;
  params.colorFormat = fmt;
  params.payloadType = NvBufferPayload_SurfArray;

  int fd = -1;
  auto ret = NvBufferCreateEx(&fd, &params);
  if (ret == -1) {
    return false;
  }

  ret = NvBufferGetParams(fd, &buffer);
  if (ret == -1){
    return false;
  }
  return true;
}

// Fill YUV420 buffer
bool FillBuffer(NvBufferParams& buffer) {
  assert(buffer.pixel_format = NvBufferColorFormat_YUV420);
  uint8_t values[] = {'Y', 'U', 'V'};
  for (uint32_t pidx = 0; pidx < buffer.num_planes; pidx++){
    void* ptr = nullptr;
    auto ret = NvBufferMemMap(buffer.dmabuf_fd, pidx, NvBufferMem_Read_Write, &ptr);
    if (ret == -1){
      std::cerr << "Failed to map buffer" << std::endl;
      return false;
    }
    NvBufferMemSyncForCpu(buffer.dmabuf_fd, pidx, &ptr);

    uint8_t* data = reinterpret_cast<uint8_t*>(ptr);
    for (uint32_t row = 0; row < buffer.height[pidx]; row++){
      std::fill(data, data + buffer.width[pidx], values[pidx]);
      data += buffer.pitch[pidx];
    }
    NvBufferMemSyncForDevice(buffer.dmabuf_fd, pidx, &ptr);
  }
  return true;
}

// Dump OutputBuffer
bool DumpBuffer(const std::string& filename, NvBufferParams& buffer){
  std::ofstream output(filename);
  for (uint32_t pidx = 0; pidx < buffer.num_planes; pidx++){
    void *ptr = nullptr;
    auto ret = NvBufferMemMap(buffer.dmabuf_fd, pidx, NvBufferMem_Read_Write, &ptr);
    if (ret == -1){
      std::cerr << "Failed to map output buffer" << std::endl;
      return false;
    }
    NvBufferMemSyncForCpu(buffer.dmabuf_fd, pidx, &ptr);

    char* data = reinterpret_cast<char*>(ptr);
    for (uint32_t row = 0; row < buffer.height[pidx]; row++){
      output.write(data, buffer.width[pidx]);
      data += buffer.pitch[pidx];
    }
  }
  return true;
}

int main(int argc, char* argv[]){
  NvBufferParams input{};
  NvBufferParams output{};
  if (!CreateBuffer(1280, 720, NvBufferColorFormat_YUV420, input)){
    return EXIT_FAILURE;
  }
  if (!CreateBuffer(1280, 720, NvBufferColorFormat_YUV420, output)){
    return EXIT_FAILURE;
  }

  NvBufferTransformParams tparam{};
  auto ret = NvBufferTransform(input.dmabuf_fd, output.dmabuf_fd, &tparam);
  if (ret != 0){
    return ret;
  }
  if (!DumpBuffer("output.bin", output)){
    return EXIT_FAILURE;
  }
  return 0;
}
